# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import User

from webadmin.backup.models import Backup
from webadmin.system.models import Ldap
from webadmin.tools import Task

def update_context():
    """
    Update context dictionary with
    common informations used to renderize
    HTML templates.
    """

    info_text = 'Cria um backup das principais informações do sistema, incluindo as de usuários. A restauração sempre usará o backup mais recente.'

    try:
        bkp_all = Backup.objects.all()
        total = bkp_all.count()
        task = Task()
        context= {'total': total, 'icon': 'drive-optical.png', 'info_text': info_text}
        if total <=1:
            context.update({'modulo': 'Backup',
                            'extra': 'criado',
                            'backups': bkp_all,
                            })
        else:
            context.update({'modulo': 'Backups',
                            'extra': 'criados',
                            'backups': bkp_all,
                            })
    except Backup.DoesNotExist:
        context = {}
    
    return context
            

@login_required
def list(request):
    """
    List all backups
    """

    context = {}

    if request.method == 'POST':
        context = new_backup(request)

    context.update(update_context())
    return render_to_response('backup/backup_list.html', context,
                              context_instance=RequestContext(request))

@login_required
def new_backup(request):
    """
    Create a new backup file with
    settings defined in tools.py
    """

    import datetime
    date = datetime.datetime.now()

    context = {}

    try:
        password = Ldap.objects.all()[0].passwd
        task = Task()
        backup = task.newBackup(password)

        if backup == 'Permission denied':
            errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
            context.update({'errmsg': errmsg})
            return context

        db_bkp = Backup()
        db_bkp.date = date
        db_bkp.save()

        context = {'msg': u'Backup criado com sucesso.'}
        context.update(update_context())
        return context
    except:
        context = {'errmsg': u'Falha ao criar backup, nehuma senha definida para o LDAP.'}
        context.update(update_context())
        return context

@login_required
def restore_backup(request):
    """
    Restore only the last backup.
    """

    context = {}
    try:
        task = Task()
        if request.method == 'POST':
            backup = task.restoreBackup()
            context = {'msg': u'Restaurado'}
            context.update(update_context())
            msgok = {'msg': u'Restaurado'}
            return render_to_response('backup/backup_list.html', context,
                                      context_instance=RequestContext(request))
        else:
            context = {'attmsg': u'ATENÇÃO: Qualquer modificação feita após o último backup não estará disponível após este processo!'}
            context.update(update_context())
            return render_to_response('backup/backup_restore.html', context,
                                      context_instance=RequestContext(request))
    except Backup.DoesNotExist:
        context = update_context()
        return render_to_response('backup/backup_list.html', context,
                                  context_instance=RequestContext(request))
