# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required

from django.contrib.auth.models import User
from webadmin.groups.models import Group
from webadmin.groups.forms import GroupForm
from webadmin.users.forms import FormTimer

from webadmin.tools import *
import commands, time

def update_context():
    """
    Update context dictionary with
    common data used to renderize
    HTML templates.
    """

    try:
        """
        List all groups
        """

        g_all = Group.objects.all()
        users = [g.user.all() for g in g_all]
        total = g_all.count()
        context = {'total': total}
        context.update(is_active())
        if total == 1:
            context.update({'modulo': 'Grupo',
                            'extra': 'cadastrado',
                            'grupos': g_all,
                            'usuarios': users,
                            })
        else:
            context.update({'modulo': 'Grupos',
                            'extra': 'cadastrados',
                            'grupos': g_all,
                            'usuarios': users,
                            })

    except User.DoesNotExist:
        context = {}

    return context

def check_status(status, name, context, request):
    if status == 'Permission denied':
        errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
        context.update(update_context())
        context.update({'errmsg': errmsg})
        return render_to_response('group/list.html', context,
                                  context_instance=RequestContext(request))
    if status == 'No such file or directory':
        errmsg = u'Arquivo de tarefas inexistente!'
        context.update(update_context())
        context.update({'errmsg': errmsg})
        return render_to_response('group/list.html', context,
                                  context_instance=RequestContext(request))
    if status == 1:
        # Check if group exists
        # but don't check if LDAP is running.
        if(commands.getstatusoutput('grep %s /etc/group' % name)[0] != 0):
            errmsg = u'Erro: Grupo não encontrado no sistema!'
            context.update(update_context())
            context.update({'errmsg': errmsg})
            return render_to_response('group/list.html', context,
                                      context_instance=RequestContext(request))


@login_required
def listing(request):
    """
    List all groups with it's users
    """

    context = {}
    sys = System()
    if sys.getPrinters():
        context.update({'printer_exist': True})
    context.update(update_context())
    return render_to_response('group/list.html', context,
                              context_instance=RequestContext(request))

@login_required
def include(request):
    """
    Create a new group
    """

    if request.method == 'POST':
        form = GroupForm(request.POST, request.FILES)
        context = {'form': form}

        if form.is_valid():
            group_name = form.cleaned_data['name']
            can_print = form.cleaned_data['can_print']

            task = Task()
            time.sleep(3)
            status = task.addGroup(group_name)

            if status == 'Permission denied':
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('group/include.html', context,
                                          context_instance=RequestContext(request))
            if status == 'No such file or directory':
                errmsg = u'Arquivo de tarefas inexistente!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('group/include.html', context,
                                          context_instance=RequestContext(request))
            if status == 1:
                # Check if group was created by LDAP

                # ******** /!\ ********
                # NOTE: groups in LDAP doesn't appear in /etc/group!
                # you need to find another way to check this with normal user permissions!
                # ******** /!\ ********

                # if(commands.getstatusoutput('grep %s /etc/group' % group_name)[0] != 0):
                #     errmsg = u'Erro durante a criação do grupo.'
                #     context.update(update_context())
                #     context.update({'errmsg': errmsg})
                #     return render_to_response('group/include.html', context,
                #                               context_instance=RequestContext(request))

                form.save()
                groups = Group.objects.all()
                users_objs = groups.get(name=group_name).user.all()
                users = [ u.username for u in users_objs ]	# User list for current group
                for user in users:
                    task.addToGroup(user, group_name)

                # Unblock this group for printing
                if can_print:
                    sys = System()
                    # If exists a configured printer
                    if sys.getPrinters():
                        printer_name = sys.getPrinters().keys()[0]
                        can_print_list = [group.name for group in groups if group.can_print]
                        task.togglePrinting(printer_name, can_print_list)

                mensagem = u'Grupo %s criado com sucesso.' % group_name
                context = {'msg': mensagem, 
                           'form': GroupForm()}
                context.update(update_context())
                return render_to_response('group/include.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = GroupForm()
        context = {'form': form}

    context.update(update_context())
    return render_to_response('group/include.html', context,
                              context_instance=RequestContext(request))

@login_required
def alter(request, group_id):
    """
    Modify the selected group by id
    """

    group = get_object_or_404(Group, pk=group_id)
    oldname = group.name
    form = GroupForm({'nome': group.name})
    context = {'form': form}
    users = Group.objects.get(name=group.name).user.all()
    users_before = [ u.username for u in users ]

    if request.method == 'POST':
        form = GroupForm(request.POST, instance=group)
        if form.is_valid():
            grp = form.save(commit=False)
            newname = form.cleaned_data['name']
            usrs = form.cleaned_data['user']
            can_print = form.cleaned_data['can_print']

            task = Task()
            time.sleep(3)
            # Modify group name if requested
            status = task.renameGroup(oldname, newname)
            if status == 'Permission denied':
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('group/list.html', context,
                                          context_instance=RequestContext(request))
            if status == 'No such file or directory':
                errmsg = u'Arquivo de tarefas inexistente!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('group/list.html', context,
                                          context_instance=RequestContext(request))
            if status == 1:
                # Check if group exists
                # but don't check if LDAP is running.
                # if(commands.getstatusoutput('grep %s /etc/group' % oldname)[0] != 0):
                #     errmsg = u'Erro: Grupo não encontrado no sistema!'
                #     context.update(update_context())
                #     context.update({'errmsg': errmsg})
                #     return render_to_response('group/list.html', context,
                #                              context_instance=RequestContext(request))
                # Check if group was modified in LDAP
                # if((commands.getstatusoutput('grep %s /etc/group' % newname)[0] != 0) and (oldname != newname)):
                #     errmsg = u'Erro: Falha na alteração do nome do grupo.'
                #     context.update(update_context())
                #     context.update({'errmsg': errmsg})
                #     return render_to_response('group/list.html', context,
                #                               context_instance=RequestContext(request))

                grp.name = newname
                [ grp.user.add(u.id) for u in usrs ]
            
                form.save()
                form.save_m2m()
                
                groups = Group.objects.all()
                users = groups.get(name=newname).user.all()
                users_after = [ u.username for u in users ]

                glist = Util()
                status = glist.diff(users_before, users_after)
                if status['new']:
                    for user in status['new']:
                        st = task.addToGroup(user, newname)
#                        check_status(st, newname, context, request)
                if status['exc']:
                    for user in status['exc']:
                        st = task.removeFromGroup(user, newname)
#                        check_status(st, newname, context, request)

                # Block printing task
                sys = System()
                # If exists a printer configured
                if sys.getPrinters():
                    printer_name = sys.getPrinters().keys()[0]
                    can_print_list = [group.name for group in groups if group.can_print]
                    task.togglePrinting(printer_name, can_print_list)
                    context.update({'printer_exist': True})
                
                mensagem = u'Grupo %s alterado com sucesso.' % newname
                context.update({'msg': mensagem})
                context.update(update_context())
                return render_to_response('group/list.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = GroupForm(instance=group)
        context = {'form': form}

    context.update(update_context())
    return render_to_response('group/alter.html', context,
                              context_instance=RequestContext(request))

@login_required
def block(request, group_id):
    """
    Block or unblock group for printing,
    """

    context = {}
    groups = Group.objects.all()
    can_print_list = [group.name for group in groups if group.can_print]
    task = Task()
    sys = System()
    if sys.getPrinters():
        printer_name = sys.getPrinters().keys()[0]
        context.update({'printer_exist': True})
        try:
            g = get_object_or_404(Group, pk=group_id)
            if g.can_print:
                g.can_print = False
                can_print_list.remove(g.name)
                task.togglePrinting(printer_name, can_print_list)
                msg = u'Grupo %s bloqueado para impressão.' % g.name
            else:
                g.can_print = True
                can_print_list.append(g.name)
                task.togglePrinting(printer_name, can_print_list)
                msg = u'Grupo %s liberado para impressão.' % g.name
            g.save()
            context.update({'msg': msg})
            context.update(update_context())
        except User.DoesNotExist:
            pass
    
    else:
        msg = u'Não existem impressoras configuradas neste servidor.'
        context = {'errmsg': msg, 'printer_exist': False}
        context.update(update_context())

    return render_to_response('group/list.html', context,
                              context_instance=RequestContext(request))


@login_required
def remove(request, group_id):
    """
    Remove the selected group by id
    """

    try:
        groups = Group.objects.all()
        g = groups.get(id=group_id)
        task = Task()
        time.sleep(3)
        status = task.removeGroup(g)
        context = {}
        if status == 'Permission denied':
            errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
            context.update(update_context())
            context.update({'errmsg': errmsg})
            return render_to_response('group/list.html', context,
                                      context_instance=RequestContext(request))
        if status == 'No such file or directory':
            errmsg = u'Arquivo de tarefas inexistente!'
            context.update(update_context())
            context.update({'errmsg': errmsg})
            return render_to_response('group/list.html', context,
                                      context_instance=RequestContext(request))
        if status == 1:
            # Check if group exists
            # but don't check if LDAP is running.
            # if(commands.getstatusoutput('grep %s /etc/group' % g.name)[0] != 0):
            #     errmsg = u'Erro: Grupo não encontrado no sistema!'
            #     context.update(update_context())
            #     context.update({'errmsg': errmsg})
            #     return render_to_response('group/list.html', context,
            #                               context_instance=RequestContext(request))

            if g.can_print:
                sys = System()
                # If exists a configured printer
                if sys.getPrinters():
                    printer_name = sys.getPrinters().keys()[0]
                    can_print_list = [group.name for group in groups if group.can_print]
                    can_print_list.remove(g.name)
                    task.togglePrinting(printer_name, can_print_list)
                    context.update({'printer_exist': True})

            g.delete()
            mensagem = u'Grupo %s removido.' % g.name
            context.update({'msg': mensagem})
            context.update(update_context())
            return render_to_response('group/list.html', context,
                                      context_instance=RequestContext(request))
    except Group.DoesNotExist:
        pass
    
    return render_to_response('group/list.html', {},
                              context_instance=RequestContext(request))

@login_required
def timer(request, group_id):
    """
    Modify a group session time by id.
    """

    group = get_object_or_404(Group, pk=group_id)
    form = FormTimer(instance=group)
    context = update_context()

    if request.method == 'POST':
        form = FormTimer(request.POST)
        context.update(update_context())
        if form.is_valid():
            time = form.cleaned_data['time']
            unlimited = form.cleaned_data['time_flag']
            if unlimited:
                group.time = 0;
                group.time_flag = True
                msg = 'Tempo de sessão ilimitado para o grupo %s' % group
            else:
                group.time = time
                group.time_flag = False
                msg = 'Tempo de sessão do grupo %s alterado para %s minutos' % (group, time)

            group.save()
            context.update(update_context())
            context.update({'msg': msg})
            return render_to_response('group/list.html', context,
                                      context_instance=RequestContext(request))
            
    context.update({'form': form, 'group': group})
    return render_to_response('group/session.html', context,
                              context_instance=RequestContext(request))
