# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django import forms
from django.forms.widgets import TextInput, Textarea
from django.contrib.localflavor.br.forms import BRPhoneNumberField, BRStateSelect, BRZipCodeField
from webadmin.system.models import Info

class FormAdmPasswd(forms.Form):
    """
    Create a form field to enter a new admin password.
    """

    passwd = forms.CharField(widget=forms.PasswordInput, label="Nova senha", required=True)
    password2 = forms.CharField(widget=forms.PasswordInput, label="Confirmar nova senha")

    def clean_passwd(self):
        if self.data['passwd'] != self.data['password2']:
            raise forms.ValidationError('As senhas não conferem!')
        return self.data['passwd']  

class FormRegistration(forms.ModelForm):
    """
    Create a form registration for telecentro.
    """

#    CHOICES = (('GESAC_SATELITE', 'GESAC_SATELITE',), ('OUTRA', 'Outra',))
#    connection = forms.ChoiceField(label="Conexão", choices=CHOICES, widget=forms.Select)
    state = forms.CharField(label="Estado", widget=BRStateSelect())
    city = forms.CharField(label="Município", widget=forms.Select)
    zipcode = BRZipCodeField(
        label="CEP",
        error_messages={'invalid': u'O CEP deve ser no formato XXXXX-XXX'})

    # Dummy hidden field to resolve the city problem, that was not set correctly.
    # So, the javascript loads this field, and set it in the SelectField.
    # TODO: Modify to use the new HTML5 feature: localstorage().
    dummy = forms.CharField(
        label='',
        widget=forms.TextInput(attrs={'hidden':'hidden'}),
        required=False
        )

    class Meta:
        model = Info
        exclude = ('active', 'datasid')
        widgets = {
            'neighborhood': TextInput(attrs={'onchange': 'getFullAddress()'}),
            'geolocation': TextInput(attrs={'readonly': 'readonly'}),
            'superid': TextInput(attrs={'readonly': 'readonly'}),
            'complement': Textarea(attrs={'style': 'height: 40px;'}),
            }

class FormPrinter(forms.Form):
    """
    Create a dropdown list with the printers availiable in telecentro.
    """

    CHOICES = (('Samsung', 'Samsung ML-2850D',), ('Xerox', 'Xerox Phaser 3125N',))
    printers = forms.ChoiceField(choices=CHOICES, label="Impressora")

class FormLocation(forms.Form):
    """
    Display a form to activate an pre registered telecentro.
    """

    state = forms.CharField(label="Estado", widget=BRStateSelect())
    city = forms.CharField(label="Município", widget=forms.Select)

class FormTelecentro(forms.Form):
    """
    Populate a choice field with available telecentros
    """

    def __init__(self, choices, *args, **kwargs):
        super(FormTelecentro, self).__init__(*args, **kwargs)
        self.fields['telecentro'].choices = choices

    telecentro = forms.ChoiceField()

class FormInfo(forms.ModelForm):
    """
    Create a form to display data from registered telecentro.
    """

    def __init__(self, *args, **kwargs):
        super(FormInfo, self).__init__(*args, **kwargs)
        for name, field in self.fields.items():
            field.widget.attrs['readonly'] = 'yes'

    #superid = forms.CharField(label="ID ponto")
    complement = forms.CharField(label="Endereço", widget=Textarea(attrs={'style': 'height: 40px;'}))
    class Meta:
        model = Info
        exclude = ('active', 'registered', 'street', 'number', 'superid')
        widgets = {
            'beneficiary': Textarea(attrs={'style': 'height: 40px;'}),
            'name': Textarea(attrs={'style': 'height: 40px;'}),
            }

