# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib import messages
from django.contrib.auth.models import User
from django.http import HttpResponseRedirect

from webadmin.system.models import Info
from webadmin.groups.models import Group
from webadmin.system.forms import *
from webadmin.tools import *
from webadmin.tasks import client_information

def update_context():
    """
    Update the context dictonary with
    commom informations to be rendered in the HTML template
    """

    context = {'modulo': 'Telecentro',
               'extra': 'Telecentros.BR',
               }
    context.update(is_active())

    try:
        telecentro = Info.objects.get()
        context.update({'telecentro': telecentro})
    except Info.DoesNotExist:
        pass

    return context

@login_required
def clients(request):
    """
    Function to render a list of connected machines and users in the server.
    """

    s = SSH()

    context = {}
    context.update(update_context())

    clients = s.client_information()

    context.update({'modulo': 'Gerenciamento',
                    'extra': 'Clientes',
                    'clients': clients,
                    })

    if not context['active']:
        return render_to_response('system/warning.html', context,
                                  context_instance=RequestContext(request))
    else:
        return render_to_response('system/clients.html', context,
                                  context_instance=RequestContext(request))

@login_required
def shutdown(request, host, user):
    """
    Function to turn off a terminal and render a message to webadmin page.
    """

    try:
        alertmsg = "Terminal será desligado em 5 segundos."
        s = SSH()
        conn = s.getConnection(host)
        if conn:
            s.showMessage(conn, user, alertmsg)
            s.turnOFF(conn)
            msg = u'Terminal desligado com sucesso'
            messages.success(request, msg)
        else:
            msg = u'Não foi possivel estabelecer conexão com o terminal'
            messages.error(request, msg)

    except Exception:
        errmsg = u'Erro: Problemas durante o desligamento do terminal'
        messages.error(request, msg)

    return HttpResponseRedirect('/system/clients/')

@login_required
def logoff(request, host, user):
    """
    Function to log off a user and render a message to webadmin page.
    """

    try:
        alertmsg = "Sua sessão será encerrada em 5 segundos."
        s = SSH()
        conn = s.getConnection(host)
        if conn:
            s.showMessage(conn, user, alertmsg)
            s.logOFF(conn, user)
            msg = u'Usuário desconectado com sucesso'
            messages.success(request, msg)
        else:
            errmsg = u'Não foi possível estabelecer conexão com o usuário'
            messages.error(request, msg)

    except Exception:
        errmsg = u'Erro ao desconectar o usuário'
        messages.error(request, errmsg)

    return HttpResponseRedirect('/system/clients/')

@login_required
def printer(request):
    """
    Function to render a form to select a printer to install
    or a list with information about an installed printer.
    """

    info_text = 'Aqui você pode adicionar e configurar uma impressora conectada neste telecentro a partir de uma lista pré-selecionada. Tenha certeza de selecionar a impressora correta na lista ao lado de acordo com o seu modelo!'

    context={}
    context.update(update_context())
    print context
    s = System()
    p_info = s.getDests()
    p_name = []
    for name, instance in p_info.keys():
        p_name.append(name)

    if request.method == 'POST':
        form = FormPrinter(request.POST)
        if form.is_valid():
            printer = form.cleaned_data['printers']
            # printer = 1 > Samsung ML-2850D
            # printer = 2 > Xerox Phaser 3125N
            status = s.addPrinter(printer)
            # status error codes:
            # -1: invalid printer
            #  0: not found
            #  1: ok
            msg = ''
            errmsg = ''
            if status == -1:
                errmsg = 'Impressora inválida.'
            elif status == 0:
                errmsg = 'Impressora não encontrada. Tente religar sua impressora e verificar sua conexão na porta correta do roteador.'
            elif status == 1:
                msg= 'Impressora %s configurada.' % printer
                p_info = s.getDests()
            elif status == 2:
                errmsg = 'Ocorreu um erro durante a instalação da impressora. Tente novamente mais tarde.'

        context = {'form': form, 'msg': msg, 'errmsg': errmsg}
    else:
        form = FormPrinter()
        context = {'form': form}

    context.update(update_context())
    context.update(
        {'modulo': 'Gerenciamento',
         'extra': 'Impressoras',
         'icon': 'printer.png',
         'printers_info': p_info,
         'printers_name': p_name,
         'info_text': info_text,
         })

    if not context['active']:
        return render_to_response('system/warning.html', context,
                                  context_instance=RequestContext(request))
    else:
        return render_to_response('system/hardware.html', context,
                                  context_instance=RequestContext(request))

@login_required
def printerDelete(request, printername):
    s = System()
    p = s.removePrinter(printername)
    l = s.getDests()

    info_text = 'Aqui você pode adicionar e configurar uma impressora conectada neste telecentro a partir de uma lista pré-selecionada. Tenha certeza de selecionar a impressora correta na lista ao lado de acordo com o seu modelo!'

    context = {}
    context.update(update_context())
    context.update(
        {'modulo': 'Gerenciamento',
         'extra': 'Impressoras',
         'icon': 'printer.png',
         'printers': l,
         'info_text': info_text,
         })
    return redirect('/system/hardware')
    #return render_to_response('system/hardware.html', context,
    #                              context_instance=RequestContext(request))

@login_required
def admin_password(request):
    """
    Function to render a form to change the admin password.
    """

    context = {}
    context.update(update_context())
    # if context['active']:
    #     context.update({'registered': Info.objects.all().count()})
    # context.update(update_context())
    admin_username = request.user
    try:
        admin_obj = User.objects.get(username__exact=admin_username)
    except User.DoesNotExist:
        context.update({'errmsg': 'Erro: Usuário "%s" não existe.' % admin_username})
        return render_to_response('system/adminpasswd.html', context,
                                  context_instance=RequestContext(request))

    if request.method == 'POST':
        form = FormAdmPasswd(request.POST)
        context ={'form': form}
        context.update(update_context())
        if form.is_valid():
            passwd = form.cleaned_data['passwd']

            # Create a task to be managed by a daemon,
            # to change admin password.
            task = Task()
            ret = task.changePassword(admin_username, passwd)    # return 1 or IOError
            if ret != 1:    # If IOError(permission, file not exist, etc...)
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg,
                                'adm': admin_obj})
                return render_to_response('system/adminpasswd.html', context,
                                          context_instance=RequestContext(request))
            else:
                admin_obj.set_password(passwd);
                admin_obj.save()
                context.update({'modulo': 'Segurança',
                                'extra': 'senha admin.',
                                'msg': 'Senha alterada com sucesso. Ela terá efeito em sua próxima sessão.',
                                'adm': admin_obj,})
                return render_to_response('system/adminpasswd.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = FormAdmPasswd()

    context.update(
        {'modulo': 'Segurança',
         'extra': 'senha admin.',
         'icon': 'keyring.png',
         'adm': admin_obj,
         'form': form,
         }
        )

    if not context['active']:
        return render_to_response('system/warning.html', context,
                                  context_instance=RequestContext(request))
    else:
        return render_to_response('system/adminpasswd.html', context,
                                  context_instance=RequestContext(request))

@login_required
def register(request):
    """
    Register a new 'telecentro' if they are not in the list.
    """

    util = Util()
    superid = util.get_hash()

    if request.method == 'POST':
        form_register = FormRegistration(request.POST, request.FILES)
        context = {'form_register': form_register}

        if form_register.is_valid():
            register = form_register.save(commit=False)

            name = form_register.cleaned_data['name']
            location = form_register.cleaned_data['geolocation']
            register.superid = superid
            register.active = True
            register.save()
            message = u'Telecentro %s registrado.' % name

            task = Task()
            task.setGeoLocation(location)

            context = {'msg': message}
            context.update(update_context())
            return render_to_response('system/telecentro_display.html', context,
                                      context_instance=RequestContext(request))

    else:
        form_register = FormRegistration(initial={'superid': superid})
        context = {'form_register': form_register}

    context.update(update_context())
    return render_to_response('system/telecentro_register.html', context,
                              context_instance=RequestContext(request))

@login_required
def webcam(request):
    """
    Render a page to display a link for a ip cam control software.
    """

    info_text = 'Link para acesso à página de gerenciamento da ipcam.'
    context = update_context()
    ipcam = '192.168.0.10'

    import commands
    status = commands.getstatusoutput('ping -c3 %s' % ipcam)[0]
    if status:
        message = 'Camêra não conectada ou inexistente no endereço %s' % ipcam
        context.update({'errmsg': message})

    context.update(
        {'modulo': 'Gerenciamento',
         'extra': 'Webcam',
         'icon': 'ipcam.png',
         'ip': ipcam,
         'info_text': info_text,
         })
    return render_to_response('system/webcam.html', context,
                              context_instance=RequestContext(request))


def serialize(request):
    """
    Function to serialize data in pure text format
    This is part of webservice
    """

    from django.shortcuts import HttpResponse
#    from django.core import serializers
#    from webadmin.system.models import Info

    # Serialize only the telecentro superID
    #data = serializers.serialize('xml', Info.objects.all())#, fields=('superid'))

    try:
        info = Info.objects.get()

        superid = info.superid
        name = info.name
        connection = info.connection
        beneficiary = info.beneficiary
        responsible = info.responsible
        state = info.state
        city = info.city
        neighborhood = info.neighborhood
        zipcode = info.zipcode
        address = info.complement
        geolocation = info.geolocation
        users = User.objects.all().count()

        data = '''
idponto=%s
nome=%s
conexao=%s
beneficiaria=%s
responsavel=%s
estado=%s
municipio=%s
bairro=%s
cep=%s
endereco=%s
geolocalizacao=%s
usuarios=%s
''' % (superid, name, connection, beneficiary, responsible, state, city, neighborhood, zipcode, address, geolocation, users)

        return HttpResponse(data, mimetype='text/txt')
    except Info.DoesNotExist:
        data = 'Telecentro não ativado.'
        return HttpResponse(data, mimetype='text/txt')

def sshkey(request):
    """
    Function to serialize ssh keys in pure text format
    """
    from django.shortcuts import HttpResponse

    ssh = SSH()

    authorized_keys = ssh.getAuthKey() + '\n'

    return HttpResponse(authorized_keys, mimetype='text/txt')

def save(request):
    """
    Save telecentro activation data.
    """

    info = get_object_or_404(Info, id=1)
    form = FormInfo(instance=info)
    context = {'icon': 'network.png'}
    context.update(update_context())

    #msg = 'Telecentro ativado com sucesso!'

    if request.method == 'POST':
        form = FormInfo(request.POST, instance=info)
        if form.is_valid():
            info = form.save(commit=False)
            info.active = True
            info.registered = True
            info.save()
            #context.update({'form': form, 'msg': msg})
            context = is_active()
            return render_to_response('index.html', context,
                       context_instance=RequestContext(request))

    context.update({'form': form})
    return render_to_response('system/telecentro_info.html', context,
                              context_instance=RequestContext(request))


def activate(request):
    """
    Select a pre registered telecentro for activation.
    """

    city = request.GET['city']
    state = request.GET['state']
    u = Util()
    data = u.load_by_state(state, city)
    telecentros = []
    context = {}

    if not data:
        attmsg = 'Não existem telecentros na cidade escolhida.'
        context.update({'attmsg': attmsg})
        return render_to_response('system/telecentro_display.html', context,
                                  context_instance=RequestContext(request))

    if data[0].has_key('error'):
        error = data[0]['error']
        context.update({'errmsg': error})
        return render_to_response('system/telecentro_selection.html', context,
                                  context_instance=RequestContext(request))

    for i in data:
        city_id = str(i.pop('ibge')) # All belong to the same city
        telecentros.append(tuple(i.values()))
    choices = tuple(telecentros)

    location = {'city': city, 'state': state}
    context.update({'location': location,
                    'data': data})

    if request.method == 'POST':
        superid = request.POST.get('telecentro', '')
        try:
            [ info.delete() for info in Info.objects.all() ]
        except:
            pass
        info = u.get_telecentro_info(state, city_id, superid)

        if info[0].has_key('error'):
            Info.objects.all().delete()
            form = FormLocation()
            error = info[0]['error']
            context.update({'errmsg': error, 'form': form})
            return render_to_response('system/telecentro_location.html', context,
                                      context_instance=RequestContext(request))

        data = Info()
        data.active = False

        # Little sequence to validate data from MC webservice...
        info = info.pop()
        data.superid = info['point_id']
        try:
            if  info['telecenter_name'] != "":
                data.name = info['telecenter_name']
            else:
                data.name = u'Não Informado'
        except:
            data.name = u'Não Informado'

        data.connection = u'Não informado'
        data.beneficiary = u'Não informado'

        try:
            if info['telecenter_contact_name'] != "":
                data.responsible = info['telecenter_contact_name']
            else:
                data.responsible = u'Não infomado'
        except:
            data.responsible = u'Não informado'

        data.neighborhood =  u'Não informado'

        try:
            if info['telecenter_zipcode'] != None:
                data.zipcode = info['telecenter_zipcode']
            else:
                data.zipcode = u'Não informado'
        except:
            data.zipcode = u'Não informado'

        data.street = u'Não infomado'
        data.number = 0

        try:
            if info['telecenter_address'] != None:
                data.complement = info['telecenter_address']
            else:
                data.complement = u'Não informado'
        except:
            data.complement = u'Não informado'

        data.city = city
        data.state = state
        data.save()
        return redirect('/system/telecentro/save')
    else:
        form = FormTelecentro(choices=choices)
    context.update({'form': form})

    return render_to_response('system/telecentro_selection.html', context,
                              context_instance=RequestContext(request))

@login_required
def deactivate(request):
    """
    Function to deactivate a registered telecentro.
    """

    context = {}
    [ info.delete() for info in Info.objects.all() ]
    i = Info()
    i.active = False
    i.registered = False
    i.save()
    context.update({'msg': 'Telecentro desativado com sucesso'})
    context.update(is_active())

    return render_to_response('index.html', context,
                              context_instance=RequestContext(request))


@login_required
def location(request):
    """
    Function to select the telecentro location.
    """

    form = FormLocation()
    info_text = 'Ativa o telecentro de acordo com uma lista pré-cadastrada no Ministério das Comunicações, ou registra um novo telecentro.'
    context = update_context()
    context.update(
        {'form': form,
         'info_text': info_text,
         })

    return render_to_response('system/telecentro_location.html', context,
                              context_instance=RequestContext(request))
