# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

import sys, os, errno, commands, cups, time, tempfile, paramiko
from webadmin.system.models import Info
from datetime import datetime
from django.contrib.auth.models import User
from webadmin.users.models import UserProfile
from webadmin.users.forms import *
from webadmin.groups.models import Group

class System:
    """
    System routines to manipulate users and peripheral.
    """
    def getDests(self):
        """ Return a list of installed printers and information about them """
        connection = cups.Connection()
        printers = connection.getDests()
        return printers

    def getPrinters(self):
        """ Return a list of installed printers. """
        connection = cups.Connection()
        printers = connection.getPrinters()
        for printer in printers:
            ret = printers[printer]
        printers = connection.getDests()
        return printers

    def addPrinter(self, printer_name):
        """
        Configure one of the printers approved by MC.
        """
        # return error codes:
        # -1: invalid printer
        #  0: not found
        #  1: ok
        path = '/usr/share/minicom-webadmin/webadmin/support'
        if printer_name == 'Samsung':
            ppd_file = '%s/Samsung_ML-2850_Series.ppd' % path
        elif printer_name == 'Xerox':
            ppd_file = '%s/Xerox_Phaser_3125_N.ppd' % path
        else:
            return -1

        try:
            connection = cups.Connection()
            connection.addPrinter(name=printer_name, filename=ppd_file)
            connection.enablePrinter(printer_name)
            devices = connection.getDevices(include_schemes=['dnssd'])

            uri = ''
            keys = devices.keys()
            for i in keys:
                if i.find(printer_name) > 0 and i.find('ipp') > 0:
                    uri = i
            if not uri:
                connection.deletePrinter(printer_name)
                return 0

            connection.setPrinterDevice(printer_name, uri)
            connection.acceptJobs(printer_name)
            return 1
        except:
            return 2

    def removePrinter(self, printer_name):
        connection = cups.Connection()
        connection.deletePrinter(printer_name)

    def clients(self):
        """
        Get local network interface and list machines connected to it.
        """
        tmp = commands.getoutput('/bin/cat /etc/default/isc-dhcp-server | cut -f2 -d"="')
        try:
            interface = tmp.split('"')[1]
            raw = commands.getoutput('/usr/sbin/arp -ni %s | sed 1d' % interface)
            raw_list = raw.split('\n')
            clist = []
            labels = ['ip', 'type', 'mac', 'flags', 'iface',
                        'session_time', 'online_user']

            def inRange(list):
                """
                Return only the addresses in the range
                192.168.0.11 and 192.168.0.100
                Addresses out of this range are devices
                like printer, router, ipcam.
                """
                ip = list[0]
                final = ip.split('.').pop()
                return (int(final) > 10 and int(final) < 101)

            if raw:
                for i in raw_list:
                    aux = []
                    clients = {}
                    l = i.split(' ')
                    for c in l:
                        if c:
                            aux.append(c)
                    aux.append('-')
                    aux.append('-')
                    if inRange(aux):
                        clients.update(zip(labels, aux))
                        # Test if the information gave by "arp -ni" is correct,
                        # if 'online_user' isn't a key, the information is wrong
                        # and will not be used.
                        if "online_user" in clients:
                            clist.append(clients)
                return clist
            return 0

        except IndexError:
            return 0

    def userTime(self, username):
        """
        Get the longest time in groups or 0 (i.e no limit)
        or takes the user time if user belongs to no group
        """

        try:
            user = User.objects.get(username=username)
            profile = user.get_profile()
            groups = Group.objects.filter(user=user)
            g_times = [group.time for group in groups]

            if(g_times):
                if 0 in g_times:
                    timer = 0
                else:
                    timer = max(g_times)
            else:
                timer = profile.time

        except User.DoesNotExist:
            timer = '(indeterminado)'

        return timer

    def sessionTime(self, username, time):
        """ Get the remaning session time of the user. """

        if username != 'monitor':
            loginTime = time.split(':')
            loginTime = datetime.now().replace(
                        hour=int(loginTime[0]),
                        minute=int(loginTime[1]))
            timeNow = datetime.now()

            session = int((timeNow - loginTime).total_seconds()) / 60

            timer = self.userTime(username)
            if timer == '(indeterminado)':
                return timer

            # Check if the user.time is 0. If is, his remaning time is
            # unlimited and the function should return False.
            if timer:
                sTime = timer - session
                if sTime < 0:
                    return 'Time Out'
                return str(sTime) + ' min'

        return 'Ilimitado'

    def check_database(self):
        """ Check if the user database is correct and make a update if not. """

        task = Task()
        wb_users = []
        msg = ""
        db_path = '/usr/share/minicom-webadmin/webadmin/webadmin.db'
        # Get all users created in webadmin.
        cmd = "sqlite3 %s 'SELECT * FROM auth_user' | tr '|' ' ' | awk '{print $2}'" % db_path
        webadmin = commands.getstatusoutput(cmd)
        task.getUsers()
        time.sleep(1)
        # Get all users created in ldap.
        f = open('/tmp/ldap_users', 'r')
        ldap_users = f.read()[:-1]
        if not webadmin[0]:
            for user in webadmin[1].split('\n'):
                wb_users.append(user)
            for l_user in ldap_users.split():
                # Check if the ldap user is in webadmin database.
                if l_user not in wb_users:
                    # Add user in webadmin database.
                    form_address = FormAddress()
                    form_profile = FormUser()
                    user_profile = form_profile.save(commit=False)
                    user_address = form_address.save(commit=False)

                    system_user = User.objects.create_user(username = l_user,
                                                            email = "",
                                                            password = "telecentro")
                    system_user.is_active = False
                    system_user.is_staff = False
                    system_user.first_name = ""
                    system_user.last_name = ""
                    system_user.save()

                    user_profile.user_id = system_user.id
                    user_profile.save()

                    user_address.user_id = user_profile.user_id
                    user_address.save()

                    # Change user password in ldap.
                    task.changePassword(l_user, "telecentro")

                    msg = "Database foi atualizada com sucesso."
        else:
            msg = "Erro ao atualizar a database."

        if not msg:
            msg = "Database nao precisa ser atualizada"

        return msg

class SSH:
    """ Execute SSH tasks to manipulate the clients connected to the server """

    def client_information(self):

        s = System()
        hosts = []
        onList = {}

        try:
            clients = s.clients()

            if not clients:
                clients = []

            for client in clients:
                hosts.append(client["ip"])

            onList = self.onlineUser(hosts)

            if onList:
                for client in clients:
                    if client["ip"] in onList:
                        info = onList[client["ip"]].split()
                        client["online_user"] = info[0]
                        time = s.sessionTime(info[0], info[1])
                        client["session_time"] = time
                        if time == 'Time Out':
                            conn = self.getConnection(client["ip"])
                            self.logOFF(conn, info[0])
            return clients

        except:
            clients = []
            return clients


    def getConnection(self, host):
        """" Connect to the each terminal online in the server """
        paramiko.util.log_to_file("/var/log/paramiko.log")
        try:
            client = paramiko.SSHClient()
            client.set_missing_host_key_policy(
                    paramiko.AutoAddPolicy())
            client.connect(host,
                    username='root',
                    key_filename='/usr/share/minicom-webadmin/ssh/id_dsa_minicom')
        except Exception:
            # Can't connect to the host.
            return None

        return client

    def onlineUser(self, hosts):
        """ Get the user connected to each terminal online in the server """

        onlineList = {}

        for host in hosts:
            conn = self.getConnection(host)
            if not conn:
                continue

            try:
                stdin, stdout, stderr = conn.exec_command('who')
            except:
                continue

            stdin.close()

            for line in stdout.read().splitlines():
                user = line.split()
                # Get the username and the login time of the user.
                if user[1] == 'tty7':
                    onlineList[host] = user[0] + ' ' + user[3]
        return onlineList

    def getAuthKey(self):
        """ Get the ssh public key """
        authKey = open('/usr/share/minicom-webadmin/ssh/id_dsa_minicom.pub').read()
        return authKey

    def showMessage(self, conn, user, msg):
        """ Show a message to client """

        if user != "-":
            cmd = 'sudo -u ' + str(user) + ' DISPLAY=:0.0 notify-send "' + msg + '"'
            stdin, stdout, stderr = conn.exec_command(cmd)
            stdin.close()

            time.sleep(5)

        return 0

    def turnOFF(self, conn):
        """ Turn off the terminal """

        stdin, stdout, stderr = conn.exec_command('shutdown -h now')
        stdin.close()
        conn.close()

        return 0

    def logOFF(self, conn, user):
        """ Finish the user session """

        stdin, stdout, stderr = conn.exec_command('slay -clean ' + user)
        stdin.close()
        conn.close()

        return 0

class Task:
    """ Execute system tasks that are not allowed for www-data user. """

    def write_to_file(self, values, exit=True):
        taskdir = '/daemon-files/'
        """
        Writes a sequence of commands to be executed in a file that is monitored by a daemon.
        """

        try:
            filename = tempfile.mkstemp(suffix='.mc', dir=taskdir)[1]
        except OSError, e:
            return os.strerror(e.errno)

        if exit:
            values += "\nexit"

        try:
            fd = open(filename, 'a')
            fd.write('%s\n' % values.encode('utf-8'))
            fd.close()
        except IOError, e:
            return os.strerror(e.errno)

        return 1

    def random_filename(self):
        """ Generate a random filename. """

        filename = tempfile.mkstemp('.ldif', 'minicom-webadmin')[1]
        return filename
    def addSSHKey(self):
        """ Performs ssh key creation in server. """

        data = "bash minicom-webadmin_ssh_key.sh"
        return self.write_to_file(data)

    def addUser(self, name, passwd, quota=0):
        """ Performs user creation in ldap. """

        group = 'users'
        squota = int(quota) - 500
        data = ('ldapadduser %s %s' % (name, group)) + '\n'
        data += ('ldapsetpasswd %s %s' % (name, passwd))

        return self.write_to_file(data)

    def alterUser(self, oldname, newname, quota=0):
        """ Performs user modification in ldap. """
        # precisa saber onde atribuir o valor da cota
        squota = int(quota) - 500
        data0 = []
        if oldname != newname:
            data0 = ('ldaprenameuser %s %s' % (oldname, newname))
        if data0:
            return (self.write_to_file(data0))
        return 1

    def blockUser(self, name):
        """ Performs user block. """

        filename = self.random_filename()
        f = open(filename, "w")

        ldap = "dn: uid=%s,ou=People,dc=telecentro,dc=net" % name
        ldap += "\nchangetype: modify"
        ldap += "\nadd: pwdAccountLockedTime"
        ldap += "\npwdAccountLockedTime: 000001010000Z"

        f.write(ldap)
        f.close()

        data = "ldapmodify -D cn=admin,dc=telecentro,dc=net "
        data += "-y /etc/ldap-password/ldap-password.secret "
        data += "-x -h localhost -f %s" % filename
        data += "\nrm %s" % filename

        return self.write_to_file(data)

    def unblockUser(self, name):
        " Performs user unblock. """

        filename = self.random_filename()
        f = open(filename, "w")

        ldap = "dn: uid=%s,ou=People,dc=telecentro,dc=net" % name
        ldap += "\nchangetype: modify"
        ldap += "\ndelete: pwdAccountLockedTime"

        f.write(ldap)
        f.close()

        data = "ldapmodify -D cn=admin,dc=telecentro,dc=net "
        data += "-y /etc/ldap-password/ldap-password.secret "
        data += "-x -h localhost -f %s" % filename
        data += "\nrm %s" % filename

        return self.write_to_file(data)

    def removeUser(self, name):
        """ Performs user exclusion in ldap. """
        data = ('ldapdeleteuser %s' % name) + '\n'
        data += ('rm -rf /export/users/%s' % name)
        return self.write_to_file(data)

    def addGroup(self, group):
        """ Performs group creation in ldap. """
        data = ('ldapaddgroup %s' % group)
        return self.write_to_file(data)

    def removeGroup(self, group):
        """ Perform removal of the group in ldap. """
        data = ('ldapdeletegroup %s' % group.name)
        return self.write_to_file(data)

    def renameGroup(self, old_name, new_name):
        """ Performs the modification of the group name in ldap. """
        data = ('ldaprenamegroup %s %s' % (old_name, new_name))
        return self.write_to_file(data)

    def addToGroup(self, name, group):
        """ Add a user in an existing ldap group. """
        data = ('ldapaddusertogroup %s %s' % (name, group))
        return self.write_to_file(data)

    def removeFromGroup(self, name, group):
        """ Removes a user from an group in ldap. """
        data = ('ldapdeleteuserfromgroup %s %s' % (name, group))
        return self.write_to_file(data)

    def changePassword(self, username, newpasswd):
        """ Changes the password of a user in ldap. """
        data = ('ldapsetpasswd %s %s' % (username, newpasswd))
        return self.write_to_file(data)

    def getUsers(self):
        """ Get all users in ldap """
        data = ("USERS=$(slapcat | grep uid: | awk '{print $2}')") + '\n'
        data += ("echo $USERS > /tmp/ldap_users")
        return self.write_to_file(data)

    def blockUrl(self, keyword, block):
        """ Put or remove a keyword in a blocked list on Squid. """
        block_file = '/etc/squid3/sites-bloqueados'
        if os.access(block_file, os.F_OK):
            if block:   # Block
                data = ('echo %s >> %s' % (keyword, block_file)) + '\n'
                data += ('service squid3 reload')
            else:       # Unblock
                line_number = commands.getoutput('grep -n %s %s | cut -f1 -d":"' % (keyword.encode('utf-8'), block_file))
                data = ('sed -i \'%sd\' %s' % (line_number, block_file)) + '\n'
                data += ('service squid3 reload')
            return self.write_to_file(data)
        return 0

    def readLog(self, lines, logfile):
        """
        Return the Squid log filtering the blocked addresses.
        lines: Number of lines to display
        logfile: squid3 log file localization
        """

        formatted_lines = []
        try:
            cmd = "perl -pe 's/^\d+\.\d+/localtime($&)/e;' %s | grep DENIED | tail -%s" % (logfile, lines)
            status = commands.getstatusoutput(cmd)[0]
            denied_list = commands.getstatusoutput(cmd)[1].split('\n')
            if not status:
                for k in denied_list:
                    x = k.split('  ')
                    x.remove('')
                    try:
                        x.remove('')
                    except:
                        pass

                    formatted_lines.append(x)

            return (1, formatted_lines)
        except IOError, e:
            return (0, os.strerror(e.errno))

    def togglePrinting(self, printer, groups):
        """
        Block or unblock printer permissions for groups.
        """
        # E.g.
        # unblock priter for all users:
        # lpadmin -p nome_impressora -u allow:all

        # unblock printer only for user1 and user2:
        # lpadmin -p nome_impressora -u allow:user1,user2

        # block printer for user1 and user2:
        # lpadmin -p nome_impressora -u deny:user1,user2

        # block printer for user1 and group2:
        # lpadmin -p nome_impressora -u deny:user1,@grupo2

        lista = ['@'+i for i in groups]
        groups_string = ",".join(lista)
        if groups:
            data = ('lpadmin -p %s -u allow:%s' % (printer, groups_string))
        else:
            data = ('lpadmin -p %s -u deny:all' % printer)
        return self.write_to_file(data)

    def newBackup(self, passwd):
        """ Performs a backup of /etc/squid3/sites-bloqueados, ldap base and local user base. """

        source0 = '/etc/squid3/sites-bloqueados'
        target0 = '/usr/share/minicom-webadmin/webadmin'
        target1 = '/backup/ldap_backup.ldif'

        data0 = ('rsync -a %s /backup' % source0)
        data1 = ('rsync -a /%s/webadmin.db /backup' % target0)

        # Backup of ldap database
        data2 = ('ldapsearch -x -D cn=admin,dc=telecentro,dc=net -w %s -b dc=telecentro,dc=net -LLL > %s' % (passwd, target1))

        data = data0 + '\n' + data1 + '\n' + data2
        return (data)

    def restoreBackup(self):
        """ Performs a restoration of the last backup. """

        target = '/usr/share/minicom-webadmin/webadmin'
        data0 = ('rsync -a /backup/webadmin.db %s/' % target)
        data1 = ('rsync -a /backup/sites-bloqueados /etc/squi3d/')
        data2 = ('''
service slapd stop
rm -rf /var/lib/ldap/*
service slapd start
service slapd stop
slapadd -l /backup/ldap_backup.ldif
slapindex -v
chown openldap: /var/lib/ldap/*
service slapd start
''')
        # reboot?
        data = data0 + "\n" + data1 + "\n" + data2
        return (self.write_to_file(data))

    def setGeoLocation(self, location):
        """ Write the telecentro geolocation to a JSON file after the registration. """

        file_name = '/usr/share/minicom-webadmin/webadmin/media/js/location.json'
        if len(location) > 1:
            lat = location[0]
            lng = location[1]
            data = ('sed -i \'s/"last".*/"last": {"lat":"%s", "lng":"%s"}/g\' %s' % (lat, lng, file_name))
            return self.write_to_file(data)
        else:
            return 0

class Util:
    """ Utility class for manipulating data. """

    # IBGE codes for brazilian states
    state_dict = {
        "RO": 11, "AC": 12, "AM": 13,
        "RR": 14, "PA": 15, "AP": 16,
        "TO": 17, "MA": 21, "PI": 22,
        "CE": 23, "RN": 24, "PB": 25,
        "PE": 26, "AL": 27, "SE": 28,
        "BA": 29, "MG": 31, "ES": 32,
        "RJ": 33, "SP": 35, "PR": 41,
        "SC": 42, "RS": 43, "MS": 50,
        "MT": 51, "GO": 52, "DF": 53,
        }

    def diff(self, l1, l2):
        """ Return a dictionary with diff between two lists. """

        excluded = list(set(l1) - set(l2))
        new = list(set(l2) - set(l1))
        return {'new': new, 'exc': excluded}

    def load_by_state(self, state, city):
        """ Method to load a list of cities in json format from http://telecentros.sid.mc.gov.br """

        from urllib2 import urlopen
        import json

        base_url = 'http://simmc.c3sl.ufpr.br/api/telecenters/'

        if  self.state_dict.has_key(state):
            id_ibge = str(self.state_dict[state])
            url = (base_url + state)
            try:
                fp = urlopen(url)
                data = json.load(fp)
                fp.close()
                error = False
            except:
                error = 'Erro de comunicação com o servidor, verifique sua conexão com a internet ou tente novamente mais tarde.'

            if not error:
                d = {}
                l = []
                for c in data:
                    if c['city_name'] == city.upper():
                        d = {'superid': c['telecenter_id'], 'name': c['telecenter_name'], 'ibge': c['city_id']}
                        l.append(d)
                return l
            else:
                return [{'error': error}]
        return {}

    def get_telecentro_info(self, state, city_id, superid):
        """
        Get all the telecentro info by superid.
        """

        from urllib2 import urlopen
        import json

        base_url = 'http://simmc.c3sl.ufpr.br/api/telecenters/'
        url = (base_url + state + '/' + city_id + '/' + superid)
        try:
            fp = urlopen(url)
            data = json.load(fp)
            fp.close()
            if data:
                return data
            else:
                error = 'Nenhuma informação encontrada sobre este telecentro.'
        except:
            error = 'Erro de comunicação com o servidor, verifique sua conexão com a internet ou tente novamente mais tarde.'

        if error:
            return [{'error': error}]

    def get_hash(self, superid='00000'):
        """
        Create a superID for new 'telecentros'."
        """

        import uuid, md5

        hwaddress = hex(uuid.getnode())[2:-1]
#        digest = md5.new(hwaddress).hexdigest()

        return hwaddress+superid

# Function to verify if telecenter is active and/or registered
def is_active():
    try:
        info = Info.objects.get()
        registered = info.registered
        active = info.active
        if active and registered:
            # ativado e registrado
            return {'active': True, 'registered': True}
        if active and not registered:
            # ativacao e nao registrado
            return {'active': True, 'registered': False}
        # nao ativado, tentativa de ativacao, porem cancelado
        return {'active': False, 'registered': True}
    except Info.DoesNotExist:
        # se primeiro uso, entao nao ativado, e supoe registrado
        return {'active': False, 'registered': True}
