# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django import forms
from django.contrib.auth.models import User
from webadmin.users.models import UserProfile, UserAddress
from django.contrib.localflavor.br.forms import BRPhoneNumberField, BRStateSelect, BRZipCodeField

class FormPasswd(forms.Form):
    """
    Create a form field to enter a new user password.
    """

    passwd = forms.CharField(widget=forms.PasswordInput, label="Nova senha", required=True)
    password2 = forms.CharField(widget=forms.PasswordInput, label="Confirmar nova senha")

    def clean_passwd(self):
        if self.data['passwd'] != self.data['password2']:
            raise forms.ValidationError('As senhas não conferem!')
        return self.data['passwd']

class FormTimer(forms.ModelForm):
    """
    Create a form field to set user session time.
    """

    time = forms.IntegerField(
        label="Tempo de uso(minutos)",
        min_value=60,
        initial=60,
        error_messages={'min_value': u'O tempo mínimo é de 60 minutos.'},
        )

    class Meta:
        model = UserProfile
        fields = ('time', 'time_flag',)

class FormRegister(forms.ModelForm):
    """
    Create a form for basic user registration.
    """

    username = forms.CharField(label='Usuário',)
#    first_name = forms.CharField(label='Primeiro nome', required=True)
#    last_name = forms.CharField(label='Último nome', required=True)
    password = forms.CharField(widget=forms.PasswordInput, label="Senha", required=True)
    password2 = forms.CharField(widget=forms.PasswordInput, label="Confirmar senha")

    class Meta:
        model = User
        fields = ('username', 'first_name', 'last_name', 'email', 'password')

    def clean_password(self):
        if self.data['password'] != self.data['password2']:
            raise forms.ValidationError('As senhas não conferem!')
        return self.data['password']

class FormUpdateRegister(forms.ModelForm):
    """
    Create a form for update the basic user data registration.
    """

#    username = forms.CharField(label='Usuário',
#                               widget=forms.TextInput(attrs={'disabled': 'disabled'}),
#                               )

    class Meta:
        model = User
        fields = ('first_name', 'last_name', 'email')

class FormUser(forms.ModelForm):
    """
    Create a form for user profile registration.
    """

    birth_date = forms.DateField(
        label="Data de nascimento",
        error_messages={'invalid': u'Informe uma data válida no formato dd/mm/yyyy'},
        widget=forms.DateInput(attrs={'class': 'date-pick'}, format='%d/%m/%Y'),
        required=False,
        )
    social_network_other = forms.CharField(
        label="Rede social",
        required=False,
        )
    # time = forms.IntegerField(
    #     label="Tempo de uso(minutos)",
    #     min_value=60,
    #     initial=60,
    #     error_messages={'min_value': u'kkk'},
    #     )

    class Meta:
        model = UserProfile
        exclude = ('user', 'quota', 'time', 'time_flag', 'is_blocked')

class FormAddress(forms.ModelForm):
    """
    Create a form for user address registration.
    """

    state = forms.CharField(label="Estado", widget=BRStateSelect())
    city = forms.CharField(label="Município", widget=forms.Select)
    zipcode = BRZipCodeField(
        required=False,
        label="CEP",
        error_messages={'invalid': u'O CEP deve ser no formato XXXXX-XXX'})

    # Dummy field to resolve the city problem, that was not set correctly.
    # So, the javascript loads this field, and set it in the SelectField.
    dummy = forms.CharField(
        label='',
        required=False,
        widget=forms.TextInput(attrs={'hidden':'hidden'})
        )

    class Meta:
        model = UserAddress
        exclude = ('user')

class SearchForm(forms.Form):
    """
    Search users by username, first_name or email, in this order.
    """

    keyword = forms.CharField(label=u'', max_length=100)
