# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.http import HttpResponseRedirect

from django.contrib.auth.models import User
from webadmin.system.models import Info
from webadmin.users.models import UserProfile, UserAddress
from webadmin.users.forms import *

from webadmin.tools import *
from webadmin.tasks import check_database

import os, commands, time

def update_context():
    """
    Update context dictionary with
    common informations used to renderize
    HTML templates.
    """
    try:
        """
        List all users, but administrator, 'monitor'
        """

        # Make sure that the database is updated.
        check_database()

        profiles = []
        p_all = User.objects.exclude(username__exact='monitor')
        total = p_all.count()

        for u in p_all:
            try:
                user = u.get_profile()
                profiles.append(user)
            except UserProfile.DoesNotExist:
                total -= 1

        context = {'profiles': profiles,
                   'total': total,
                   }

        if total == 1:
            context.update({'modulo': 'Usuário',
                            'extra': 'cadastrado'})
        else:
            context.update({'modulo': 'Usuários',
                            'extra': 'cadastrados'})

    except User.DoesNotExist:
        context = {}

    return context

@login_required
def listing(request):
    """
    Function to render a list all users registered in telecentro.
    """

    context = update_context()
    context.update(is_active())

    # Used to search for username, name or email
    monitor_user = User.objects.get(pk=1).username
    if request.method == 'POST':
        form = SearchForm(request.POST)
        if form.is_valid():
            keyword = form.cleaned_data['keyword']
            # The user must not be the monitor user, because he doesn't have an profile associated!
            if keyword != monitor_user:
                options = {'username': User.objects.filter(username__exact=keyword),
                           'email': User.objects.filter(email__exact=keyword),
                           'name': User.objects.filter(first_name__exact=keyword),
                           }

                user = []
                for k in options.keys():
                    if options[k]:
                        user = options[k]

                if user:
                    profiles = [ u.get_profile() for u in user ]
                    context.update({'profiles': profiles,
                                    'usr': user,
                                    'form': form,
                                   })

                else:
                    context.update({'alertmsg': u'Nenhum usuário encontrado com este critério de busca.',
                                    'form': form})
                return render_to_response('user/list.html', context,
                                          context_instance=RequestContext(request))

    form = SearchForm()
    context.update({'form': form})

    if not context['active']:
        return render_to_response('system/warning.html', context,
                                  context_instance=RequestContext(request))
    else:
        return render_to_response('user/list.html', context,
                                  context_instance=RequestContext(request))

@login_required
def include(request):
    """
    Add an user in LDAP base and in a profile database.
    """

    if request.method == 'POST':
        form_address = FormAddress(request.POST, request.FILES)
        form_profile = FormUser(request.POST, request.FILES)
        form_reg = FormRegister(request.POST, request.FILES)
        context = {'form_profile': form_profile,
                   'form_reg': form_reg,
                   'form_address': form_address,
                   }
        if form_profile.is_valid() and form_reg.is_valid() and form_address.is_valid():
            username = form_reg.cleaned_data['username']
            first_name = form_reg.cleaned_data['first_name']
            last_name = form_reg.cleaned_data['last_name']
            email = form_reg.cleaned_data['email']
            password = form_reg.cleaned_data['password']

#            quota = form_profile.cleaned_data['quota']

            # Create a task to be managed by a daemon,
            # to create a user in LDAP.
            task = Task()
#            status = task.addUser(username, password, quota)
            status = task.addUser(username, password, 0)
            if status == 'Permission denied':
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('user/include.html', context,
                                          context_instance=RequestContext(request))
            if status == 'No such file or directory':
                errmsg = u'Arquivo de tarefas inexistente!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('user/include.html', context,
                                          context_instance=RequestContext(request))
            if status == 1:
                # Check if user was created by LDAP
                time.sleep(5)
                if(commands.getstatusoutput('id %s' % username)[0] != 0):
                    errmsg = u'Erro durante a criação do usuário.'
                    context.update(update_context())
                    context.update({'errmsg': errmsg})
                    return render_to_response('user/include.html', context,
                                              context_instance=RequestContext(request))

                user_profile = form_profile.save(commit=False)
                user_address = form_address.save(commit=False)

                system_user = User.objects.create_user(username = username,
                                                       email = email,
                                                       password = password)
                system_user.is_active = False
                system_user.is_staff = False
                system_user.first_name = first_name
                system_user.last_name = last_name
                system_user.save()

                user_profile.user_id = system_user.id
                user_profile.save()

                user_address.user_id = user_profile.user_id
                user_address.save()

                msg = u'Usuário adicionado com sucesso.'
                messages.success(request, msg)
                return HttpResponseRedirect('/users/user/')

    else:
        form_profile = FormUser()
        form_reg = FormRegister()
        form_address = FormAddress()
        context = {'form_profile': form_profile,
                   'form_reg': form_reg,
                   'form_address': form_address,
                   }
    context.update(update_context())
    return render_to_response('user/include.html', context,
                              context_instance=RequestContext(request))

@login_required
def alter(request, user_id):
    """
    Modify a user selected by id.
    """
    from copy import copy
    user_profile = get_object_or_404(UserProfile, pk=user_id)
    user = get_object_or_404(User, pk=int(user_id)+1)
    address = UserAddress.objects.get(user=user)

    city = address.city
    form_reg = FormUpdateRegister(instance=user)
    form_profile = FormUser(instance=user_profile)
    form_address = FormAddress(instance=address, initial={'dummy': city})

    context = {'form_reg': form_reg,
               'form_profile': form_profile,
               'form_address': form_address,
               }

    username = user.username
    if request.method == 'POST':
        form_reg = FormUpdateRegister(request.POST, instance=user)
        form_profile = FormUser(request.POST, instance=user_profile)
        form_address = FormAddress(request.POST, instance=address)
        old_name = copy(user.username)
        if form_reg.is_valid() and form_profile.is_valid() and form_address.is_valid():
            user_profile = form_profile.save(commit=False)
            user_address = form_address.save(commit=False)
#            username = form_reg.cleaned_data['username']
            first_name = form_reg.cleaned_data['first_name']
            last_name = form_reg.cleaned_data['last_name']
            email = form_reg.cleaned_data['email']
            #password = form_reg.cleaned_data['password']
            #quota = form_profile.cleaned_data['quota']
            quota = 0
            task = Task()
##            old_quota = user.get_profile().quota
            status = task.alterUser(old_name, user.username, quota)

            if status == 'Permission denied':
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                messages.error(request, errmsg)
                return HttpResponseRedirect('/users/user/')

            if status == 'No such file or directory':
                errmsg = u'Arquivo de tarefas inexistente!'
                messages.error(request, errmsg)
                return HttpResponseRedirect('/users/user/')

            if status == 1:
                # Check if user exists in LDAP base
                # but don't check if LDAP is running.
                time.sleep(5)
                if(commands.getstatusoutput('id %s' % old_name)[0] != 0):
                    errmsg = u'Erro: Usuário não encontrado na base LDAP!'
                    context.update(update_context())
                    context.update({'errmsg': errmsg})
                    return render_to_response('user/alter.html', context,
                                              context_instance=RequestContext(request))

                # Check if username was modified in LDAP
                if((commands.getstatusoutput('id %s' % username)[0] != 0) and (old_name != username)):
                    errmsg = u'Erro: Falha na alteração do nome de usuário.'
                    context.update(update_context())
                    context.update({'errmsg': errmsg})
                    return render_to_response('user/alter.html', context,
                                              context_instance=RequestContext(request))

                system_user = User.objects.get(username__exact=user_profile.user.username)
                system_user.username = username
                system_user.first_name = first_name
                system_user.last_name = last_name
                system_user.email = email
                system_user.save()
                user_profile.save()
                form_profile.save_m2m()
                user_address.save()

                messages.success(request, u'Usuário %s alterado com sucesso.' % username)
                return HttpResponseRedirect('/users/user/')

    context.update({'username': username})
    context.update(update_context())
    return render_to_response('user/alter.html', context,
                              context_instance=RequestContext(request))

@login_required
def passwd(request, user_id):
    """
    Modify a user password by id.
    """

    user = get_object_or_404(User, pk=user_id)
    context = update_context()

    if request.method == 'POST':
        form = FormPasswd(request.POST)
        context.update({'form': form})
        if form.is_valid():
            passwd = form.cleaned_data['passwd']

            # Create a task to be managed by a daemon,
            # to change admin password.
            task = Task()
            status = task.changePassword(user.username, passwd)
            if status == 'No such file or directory':
                errmsg = u'Arquivo de tarefas inexistente!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('user/passwd.html', context,
                                          context_instance=RequestContext(request))

            if status == 'Permission denied':
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update(update_context())
                context.update({'errmsg': errmsg})
                return render_to_response('user/passwd.html', context,
                                          context_instance=RequestContext(request))
            if status == 1:
                # Check if user exists in LDAP base
                # but don't check if LDAP is running.
                time.sleep(5)
                if(commands.getstatusoutput('id %s' % user.username)[0] != 0):
                    errmsg = u'Erro: Usuário não encontrado na base LDAP!'
                    messages.error(request, errmsg)
                    return HttpResponseRedirect('/users/user/')

                user.set_password(passwd);
                user.save()
                msg = u'Senha alterada com sucesso. Ela terá efeito na próxima sessão do usuário.'
                messages.success(request, msg)
                return HttpResponseRedirect('/users/user/')
    else:
        form = FormPasswd()

    context.update(
        {'user': user,
         'form': form,})

    return render_to_response('user/passwd.html', context,
                              context_instance=RequestContext(request))

@login_required
def timer(request, user_id):
    """
    Modify a user session time by id.
    """

    user = get_object_or_404(UserProfile, pk=user_id)
    form = FormTimer(instance=user)
    context = update_context()

    if request.method == 'POST':
        form = FormTimer(request.POST)
        context.update(update_context())
        if form.is_valid():
            time = form.cleaned_data['time']
            unlimited = form.cleaned_data['time_flag']
            if unlimited:
                user.time = 0;
                user.time_flag = True
                msg = 'Tempo de sessão ilimitado para o usuário %s' % user
            else:
                user.time = time
                user.time_flag = False
                msg = 'Tempo de sessão do usuário %s alterado para %s minutos' % (user, time)

            user.save()

            messages.success(request, msg)
            return HttpResponseRedirect('/users/user/')

    context.update({'form': form, 'user': user})
    return render_to_response('user/session.html', context,
                              context_instance=RequestContext(request))


@login_required
def block(request, user_id):
    """
    Block a user from system.
    """

    try:
        user = User.objects.get(id=user_id)
        task = Task()
        status = task.blockUser(user.username)

        if status != 1:
            errmsg = u'Erro ao bloquear usuário'
            messages.error(request, errmsg)
            return HttpResponseRedirect('/users/user/')

        options = {'username': User.objects.filter(username__exact=user.username),}
        usr = options['username']

        profiles = [ u.get_profile() for u in usr ]
        profiles[0].is_blocked = True
        profiles[0].save()

        msg = u'Usuário %s bloqueado com sucesso' % user.username
        messages.success(request, msg)

        s = SSH()
        c = System()

        clients = c.clients()
        hosts = []

        if not clients:
            clients = []

        for client in clients:
            hosts.append(client["ip"])

        onUser = s.onlineUser(hosts)

        try:
            for client in clients:
                ip = client["ip"]
                if ip in onUser:
                    username = onUser[ip].split()[0]
                    if username == user.username:
                        conn = s.getConnection(ip)
                        if conn:
                            alertmsg = "A sua conta foi bloqueada e sua sessão será encerrada em 5 segundos."
                            s.showMessage(conn, user.username, alertmsg)
                            s.logOFF(conn, user.username)

        except:
            errmsg = u'Ocorreu um erro ao encerrar a sessão do usuário. O bloqueio terá efeito na próxima sessão.'
            messages.error(request, errmsg)

        return HttpResponseRedirect('/users/user/')

    except User.DoesNotExist:
            pass

    return render_to_response('user/list.html', {},
                                          context_instance=RequestContext(request))

@login_required
def unblock(request, user_id):
    """
    Unblock a user from system.
    """

    try:
        user = User.objects.get(id=user_id)
        task = Task()
        status = task.unblockUser(user.username)

        if status != 1:
            errmsg = u'Erro ao desbloquear usuário'
            messages.error(request, errmsg)
            return HttpResponseRedirect('/users/user/')

        options = {'username': User.objects.filter(username__exact=user.username),}
        usr = options['username']

        profiles = [ u.get_profile() for u in usr ]
        profiles[0].is_blocked = False
        profiles[0].save()

        msg = u'Usuário %s desbloqueado com sucesso' % user.username
        messages.success(request, msg)

        return HttpResponseRedirect('/users/user/')

    except User.DoesNotExist:
        pass

    return render_to_response('user/list.html', {},
                              context_instance=RequestContext(request))

@login_required
def remove(request, user_id):
    """
    Remove a user from system with all their data.
    """
    context = update_context()
    try:
        user = User.objects.get(id=user_id)
        task = Task()
        status = task.removeUser(user.username)

        if status == 'No such file or directory':
            errmsg = u'Arquivo de tarefas inexistente!'
            messages.error(request, errmsg)
            return HttpResponseRedirect('/users/user/')

        if status == 'Permission denied':
            errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
            messages.error(request, errmsg)
            return HttpResponseRedirect('/users/user/')

        user.delete()
        msg = u'Usuário %s removido com sucesso' % user.username
        messages.success(request, msg)
        return HttpResponseRedirect('/users/user/')

    except User.DoesNotExist:
        pass

    return render_to_response('user/list.html', {},
                              context_instance=RequestContext(request))
